@@ -16,29 +16,29 @@
 // with Gnome Weather; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-const Gio = imports.gi.Gio;
-const GLib = imports.gi.GLib;
-const GObject = imports.gi.GObject;
-const Gdk = imports.gi.Gdk;
-const Gtk = imports.gi.Gtk;
-const GWeather = imports.gi.GWeather;
+import Gio from 'gi://Gio';
+import GLib from 'gi://GLib';
+import GObject from 'gi://GObject';
+import Gtk from 'gi://Gtk';
+import Gdk from 'gi://Gdk';
+import GWeather from 'gi://GWeather';
+import Graphene from 'gi://Graphene';
 
-const Util = imports.misc.util;
+import * as Util from '../misc/util.js';
 
 // In microseconds
-const TWENTY_FOUR_HOURS = 24 * 3600 * 1000 * 1000;
+const ONE_HOUR = 60 * 60 * 1000 * 1000;
+const TWENTY_FOUR_HOURS = 24 * ONE_HOUR;
 
-var HourlyForecastBox = GObject.registerClass(class HourlyForecastBox extends Gtk.Box {
-
-    _init(params) {
-        super._init(Object.assign({
+export class HourlyForecastBox extends Gtk.Box {
+    constructor() {
+        super({
             orientation: Gtk.Orientation.HORIZONTAL,
             spacing: 0,
             name: 'hourly-forecast-box',
-        }, params));
-
-        this.get_accessible().accessible_name = _('Hourly Forecast');
+        });
 
+        this.update_property([Gtk.AccessibleProperty.LABEL], [_('Hourly Forecast')]);
         this._settings = new Gio.Settings({ schema_id: 'org.gnome.desktop.interface' });
 
         this._hourlyInfo = [];
@@ -48,94 +48,108 @@ var HourlyForecastBox = GObject.registerClass(class HourlyForecastBox extends Gt
 
     // Ensure that infos are sufficiently spaced, and
     // remove infos for the wrong day
-    _preprocess(now, infos) {
-        let ret = [];
-
-        for (let i = 0; i < infos.length; i++) {
-            let info = infos[i];
-
-            let [ok, date] = info.get_value_update();
+    _preprocess(now, forecastInfo, infos) {
+        const ret = [forecastInfo, ...infos].filter(info => {
+            let [, date] = info.get_value_update();
             let datetime = GLib.DateTime.new_from_unix_utc(date).to_timezone(now.get_timezone());
 
-            if (datetime.difference(now) <= 0)
-                continue;
+            // Show the previous hour's forecast until 30 minutes in
+            if (datetime.difference(now) <= -ONE_HOUR / 2)
+                return false;
 
             if (datetime.difference(now) >= TWENTY_FOUR_HOURS)
-                break;
+                return false;
 
-            ret.push(info);
-        }
+            return true;
+        });
 
         return ret;
     }
 
-    update(infos, tz) {
+    update(info) {
+        let forecasts = info.get_forecast_list();
+
+        let coords = info.location.get_coords();
+        let nearestCity = GWeather.Location.get_world().find_nearest_city(coords[0], coords[1]);
+        let tz = nearestCity.get_timezone();
         let now = GLib.DateTime.new_now(tz);
-        let hourlyInfo = this._preprocess(now, infos);
+
+        let hourlyInfo = this._preprocess(now, info, forecasts);
 
         if (hourlyInfo.length > 0) {
             for (let i = 0; i < hourlyInfo.length; i++) {
-                let info = hourlyInfo[i];
-                this._addHourEntry(info, tz);
+                const info = hourlyInfo[i];
+                const isNow = i === 0;
+                this._addHourEntry(info, tz, isNow);
 
                 if (i < hourlyInfo.length - 1)
                     this._addSeparator();
             }
         } else {
-            let label = new Gtk.Label({ label: _('Forecast not available'),
-                                        use_markup: true,
-                                        visible: true });
-            this.pack_start(label, true, false, 0);
+            let label = new Gtk.Label({
+                label: _('Forecast not Available'),
+                use_markup: true,
+                visible: true
+            });
+            this.prepend(label);
         }
 
         this._hourlyInfo = hourlyInfo;
     }
 
-    _addHourEntry(info, tz) {
-        let [ok, date] = info.get_value_update();
+    _addHourEntry(info, tz, now) {
+        let timeLabel;
+
+        let [, date] = info.get_value_update();
         let datetime = GLib.DateTime.new_from_unix_utc(date).to_timezone(tz);
 
-        let timeSetting = this._settings.get_string('clock-format');
-        let timeFormat = null;
+        if (now) {
+            timeLabel = _('Now');
+        } else {
+            let timeSetting = this._settings.get_string('clock-format');
+            let timeFormat = null;
 
-        if (timeSetting == '12h')
-            /* Translators: this is a time format without date used for AM/PM */
-            timeFormat = _('%l∶%M %p');
-        else
-            timeFormat = '%R';
+            if (timeSetting == '12h')
+                /* Translators: this is a time format without date used for AM/PM */
+                timeFormat = _('%l∶%M %p');
+            else
+                timeFormat = '%R';
 
-        let hourEntry = new HourEntry();
+            timeLabel = datetime.format(timeFormat);
+        }
 
-        hourEntry.timeLabel.label = datetime.format(timeFormat);
-        hourEntry.image.iconName = info.get_icon_name() + '-small';
-        hourEntry.temperatureLabel.label = Util.getTempString(info);
+        let hourEntry = new HourEntry({ info, timeLabel });
 
-        if (Util.isDarkTheme()) {
-            const color = "#f6d32d";
-            const label = "<span color=\""+ color + "\">" + hourEntry.temperatureLabel.label + "</span>";
-            hourEntry.temperatureLabel.set_markup(label);
-        };
-
-        this.pack_start(hourEntry, false, false, 0);
+        this.append(hourEntry);
 
         this._hasForecastInfo = true;
     }
 
     _addSeparator() {
-        let separator = new Gtk.Separator({ orientation: Gtk.Orientation.VERTICAL,
-                                            visible: true});
-        this.pack_start(separator, false, false, 0);
+        let separator = new Gtk.Separator({
+            orientation: Gtk.Orientation.VERTICAL,
+            visible: true
+        });
+        this.append(separator);
     }
 
     clear() {
-        this.foreach(function(w) { w.destroy(); });
+        for (const w of Array.from(this)) {
+            this.remove(w);
+        }
     }
 
     hasForecastInfo() {
         return this._hasForecastInfo;
     }
 
-    vfunc_draw(cr) {
+    vfunc_snapshot(snapshot) {
+        const allocation = this.get_allocation();
+
+        const rect = new Graphene.Rect();
+        rect.init(0, 0, allocation.width, allocation.height);
+
+        let cr = snapshot.append_cairo(rect);
         const temps = this._hourlyInfo.map(info => Util.getTemp(info));
 
         const maxTemp = Math.max(...temps);
@@ -151,7 +165,7 @@ var HourlyForecastBox = GObject.registerClass(class HourlyForecastBox extends Gt
         const width = this.get_allocated_width();
         const height = this.get_allocated_height();
 
-        const entryWidth = 75 ;
+        const entryWidth = 75;
         const separatorWidth = 1;
 
         const lineWidth = 2;
@@ -165,11 +179,11 @@ var HourlyForecastBox = GObject.registerClass(class HourlyForecastBox extends Gt
         const graphMaxY = height - lineWidth / 2 - spacing - entryTemperatureLabelHeight - spacing;
         const graphHeight = graphMaxY - graphMinY;
 
-        let [, strokeColor] = this.get_style_context().lookup_color('temp_chart_stroke_color');
+        let [, strokeColor] = this.get_style_context().lookup_color('weather_temp_chart_stroke_color');
         Gdk.cairo_set_source_rgba(cr, strokeColor);
 
         let x = 0;
-        cr.moveTo (x, graphMinY + ((1 - values[0]) * graphHeight));
+        cr.moveTo(x, graphMinY + ((1 - values[0]) * graphHeight));
 
         x += entryWidth / 2;
         cr.lineTo(x, graphMinY + ((1 - values[0]) * graphHeight));
@@ -184,12 +198,7 @@ var HourlyForecastBox = GObject.registerClass(class HourlyForecastBox extends Gt
         cr.setLineWidth(lineWidth);
         cr.strokePreserve();
 
-        let [, fillColor] = this.get_style_context().lookup_color('temp_chart_fill_color');
-
-        if (Util.isDarkTheme()) {
-            fillColor = new Gdk.RGBA();
-            fillColor.parse("rgba(248, 228, 92, 0.15)");
-        };
+        let [, fillColor] = this.get_style_context().lookup_color('weather_temp_chart_fill_color');
 
         Gdk.cairo_set_source_rgba(cr, fillColor);
 
@@ -197,31 +206,34 @@ var HourlyForecastBox = GObject.registerClass(class HourlyForecastBox extends Gt
         cr.lineTo(0, height);
         cr.fill();
 
-        super.vfunc_draw(cr);
+        super.vfunc_snapshot(snapshot);
         cr.$dispose();
-
-        return Gdk.EVENT_PROPAGATE;
     }
-});
+};
+GObject.registerClass(HourlyForecastBox);
 
-var HourEntry = GObject.registerClass({
+export const HourEntry = GObject.registerClass({
     Template: 'resource:///org/gnome/Weather/hour-entry.ui',
-    InternalChildren: ['timeLabel', 'image', 'temperatureLabel'],
-}, class HourEntry extends Gtk.Box {
 
-    _init(params) {
-        super._init(params);
+    InternalChildren: ['timeLabel', 'image', 'forecastTemperatureLabel'],
+}, class HourEntry extends Gtk.Widget {
+    constructor({ timeLabel, info, ...params }) {
+        super({ ...params });
+
+        Object.assign(this.layoutManager, {
+            orientation: Gtk.Orientation.VERTICAL,
+        });
+
+        this._timeLabel.label = timeLabel;
+        this._image.iconName = info.get_icon_name() + '-small';
+        this._forecastTemperatureLabel.label = Util.getTempString(info);
     }
 
-    get timeLabel() {
-        return this._timeLabel;
-    }
+    vfunc_unroot() {
+        [...this].forEach(child => child.unparent());
 
-    get image() {
-        return this._image;
-    }
-
-    get temperatureLabel() {
-        return this._temperatureLabel;
+        super.vfunc_unroot();
     }
 });
+
+HourEntry.set_layout_manager_type(Gtk.BoxLayout);
\ No newline at end of file
